#!/bin/bash

[[ $1 = -v ]] && VERBOSE=1 || unset VERBOSE
[[ -z $VERBOSE ]] && exec 2>/dev/null
export VERBOSE

if [[ ! -w /etc/passwd ]] ; then
  echo "Only 'root' can execute this script."
  exit 1
fi

#
# Set epages environmental variables
#
SYS_SCRIPT=/etc/sysconfig/epages6
if [[ -f $SYS_SCRIPT ]] ; then
  cp -p $SYS_SCRIPT $SYS_SCRIPT.tmp \
  && sed 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' $SYS_SCRIPT.tmp > $SYS_SCRIPT \
  && rm -f $SYS_SCRIPT.tmp
  . $SYS_SCRIPT
fi
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

die() {
  touch $EPAGES/Setup/mysql.ini
  exit 0
}

unset VOID_VAR
void_var() {
  echo "Following variable value(s) may be incorrect:"
  for i in $@ ; do echo "  $i=\"$(eval echo '$'$i)\"" ; done
  echo "Check these variable(s) in $SYS_SCRIPT and rerun:"
  echo "  $0"
  VOID_VAR=1
}

RUN_SCRIPT=/etc/default/$EPAGES_PROJECT
ls $RUN_SCRIPT >/dev/null || die

cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp \
&& sed 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' $RUN_SCRIPT.tmp > $RUN_SCRIPT \
&& rm -f $RUN_SCRIPT.tmp
. $RUN_SCRIPT

export EPAGES_INSTALL_MY_CNF
[[ -n $EPAGES_INSTALL_MY_CNF ]] && MY_CNF_ARG="-c $EPAGES_INSTALL_MY_CNF" || unset MY_CNF_ARG

get_mysql_option(){
  for group in $1 ; do
    result=$(my_print_defaults $MY_CNF_ARG $group | sed -n "s/^--$2=//p" | tail -n 1)
    [[ -n $result ]] && { echo -n $result; return; }
  done
  echo -n $3
}

#
# Read the connection settings from sysconfig file
#
[[ -n $EPAGES_MYSQL_CONNECTION ]] && EPAGES_INSTALL_MYSQL_CONNECTION=$EPAGES_MYSQL_CONNECTION
# EPAGES_INSTALL_MYSQL_CONNECTION can be either 'host;port' or 'host=host;port=port'
EPAGES_INSTALL_MYSQL_CONNECTION=$(echo $EPAGES_INSTALL_MYSQL_CONNECTION | sed -r 's,[a-z]+=,,g')

export MYSQL_CONN=${EPAGES_INSTALL_MYSQL_CONNECTION:-$EPAGES_INSTALL_MYSQL_HOST}

MYSQL_DATADIR=$(get_mysql_option "server mysqld" datadir /var/lib/mysql)
MYSQL_SOCKET=$(get_mysql_option "client mysql server mysqld" socket $MYSQL_DATADIR/mysql.sock)
MYSQL_SOCKET_ARG=-S$MYSQL_SOCKET

MYSQL_HOST=${EPAGES_INSTALL_MYSQL_HOST:-${MYSQL_CONN%;*}}
MYSQL_HOST=$(get_mysql_option "client mysql" host ${MYSQL_HOST:-$(hostname)})
[[ $MYSQL_HOST = localhost ]] && MYSQL_HOST=$(hostname)
MYSQL_PORT=${MYSQL_CONN#*;}
[[ $MYSQL_CONN = $MYSQL_PORT ]] && MYSQL_PORT=
MYSQL_PORT=$(get_mysql_option "client mysql server mysqld" port ${MYSQL_PORT:-3306})

#
# Set the mysql connection string
#
if [[ $MYSQL_PORT = 3306 ]] ; then
  EPAGES_INSTALL_MYSQL_CONNECTION=$MYSQL_HOST
  MYSQL_CONN="host=$MYSQL_HOST"
  CONNECTION_ARGS="-h $MYSQL_HOST"
else
  EPAGES_INSTALL_MYSQL_CONNECTION="$MYSQL_HOST;$MYSQL_PORT"
  MYSQL_CONN="host=$MYSQL_HOST;port=$MYSQL_PORT"
  CONNECTION_ARGS="-h $MYSQL_HOST -P $MYSQL_PORT"
fi

#
# Read the backup user credentials
#
unset MYSQL_PASS_ARG
export MYSQL_USER=${EPAGES_INSTALL_MYSQL_USER:-root}
export MYSQL_PASS=$EPAGES_INSTALL_MYSQL_PASS
MYSQL_PASS_ARG=${MYSQL_PASS:+"-p'$MYSQL_PASS'"}

#
# Test the socket connection
#
check_mysql_connection(){
  local RET
  if [[ ${@%-pepages} = $@ ]] ; then
    echo "mysql $@" >/tmp/proc.~$$~
    echo exit | eval mysql $@ 2>>/tmp/proc.~$$~
    RET=$?
    [[ -n $VERBOSE ]] && echo "-> $(cat /tmp/proc.~$$~)"
  else
    [[ -n $VERBOSE ]] && echo "mysql $@"
    echo exit | eval mysql $@
    RET=$?
    [[ $RET = 0 ]] && { MYSQL_PASS=epages; MYSQL_PASS_ARG=-pepages; }
  fi
  return $RET
}

if check_mysql_connection $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG \
   || check_mysql_connection $MYSQL_SOCKET_ARG -u$MYSQL_USER -pepages ; then
  rm -f /tmp/proc.~$$~
else
  [[ -z $VERBOSE ]] && cat /tmp/proc.~$$~
  void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
fi

#
# Database engine must be InnoDB
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="SHOW ENGINES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\s+(YES|DEFAULT)\s+' || {
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
    echo "ERROR: Statement failed, database engine InnoDB not enabled."
    die
  }
fi

#
# Grant all privileges on *.* to $MYSQL_USER
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="DELETE FROM mysql.user WHERE user = ''; FLUSH PRIVILEGES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
  STATEMENT="SELECT user,host FROM mysql.user WHERE user = '';"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG > /tmp/pid.$$
  if [[ -s /tmp/pid.$$ ]] ; then
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
    echo "ERROR: Statement failed, empty users still exist:"
    cat /tmp/pid.$$
    rm -f /tmp/pid.$$
    die
  fi
  rm -f /tmp/pid.$$
  [[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
  STATEMENT="GRANT ALL ON *.* TO '$MYSQL_USER' $IDENTIFIED_BY WITH GRANT OPTION;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
fi

#
# Test the connection based on the connection string
#
if check_mysql_connection $CONNECTION_ARGS -u$MYSQL_USER $MYSQL_PASS_ARG ; then
  rm -f /tmp/proc.~$$~
else
  [[ -z $VERBOSE ]] && cat /tmp/proc.~$$~
  void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
fi

#
# Set EPAGES_MYSQL_CONNECTION in /etc/default/epages6
#
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
if [[ $EPAGES_MYSQL_CONNECTION != $EPAGES_INSTALL_MYSQL_CONNECTION ]] ; then
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v '\"$EPAGES_INSTALL_MYSQL_CONNECTION\"' -i"
  ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v "\"$EPAGES_INSTALL_MYSQL_CONNECTION\"" -i
fi

if [[ $EPAGES_INSTALL_MYSQL_HOST != $MYSQL_HOST ]] ; then
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v '\"$MYSQL_HOST\"' -i"
  ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v "\"$MYSQL_HOST\"" -i
fi
[[ -n $VERBOSE ]] && echo "ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_CONNECTION -v '\"$EPAGES_INSTALL_MYSQL_CONNECTION\"' -i"
ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_CONNECTION -v "\"$EPAGES_INSTALL_MYSQL_CONNECTION\"" -i

[[ -z $VOID_VAR ]] || rm -f $EPAGES/Setup/mysql.ini

#
# Rewrite Backup.conf to update the datasource string and the backup user credentials
#
BACKUP_CONF="$EPAGES_CONFIG/Database.d/Backup.conf"
[[ -f $BACKUP_CONF ]] || exit 0

ini-config.sh -f $BACKUP_CONF -s Connection -k login -v "$MYSQL_USER" -i
ini-config.sh -f $BACKUP_CONF -s Connection -k datasource -v "dbi:mysql:$MYSQL_CONN" -i

ENC=$($PERL -e "use DE_EPAGES::Database::API::Connection qw(EncryptDBPassword); print EncryptDBPassword('$MYSQL_PASS')")
[[ -z $ENC ]] && { USE=passwd; DROP=encpasswd; } || { USE=encpasswd; DROP=passwd; MYSQL_PASS=$ENC; }

grep -v $DROP $BACKUP_CONF > $BACKUP_CONF.$$ && cat $BACKUP_CONF.$$ > $BACKUP_CONF
rm -f $BACKUP_CONF.$$
ini-config.sh -f $BACKUP_CONF -s Connection -k $USE -v "$MYSQL_PASS" -i

if [[ -n $EPAGES_INSTALL_MY_CNF ]] ; then
  ini-config.sh -f $BACKUP_CONF -s Connection -k MySQLConfigFile -v "$EPAGES_INSTALL_MY_CNF" -i
fi

chown ep_appl:epages $BACKUP_CONF

exit 0
